@@ -2,12 +2,12 @@
 // Chair of Social Psychology, University of Freiburg
 // Authors: Christoph Klauer and Raphael Hartmann
 
-#include "tools.h"
+#include <cmath>
+#include <thread>
 #include "pdf_fncs.h"
 #include "cdf_fncs.h"
 #include "fncs_seven.h"
-#include <cmath>
-#include <thread>
+#include "tools.h"
 
 
 
@@ -555,7 +555,7 @@ void dxPDF_old(double *t, double *a, double *v, double *w, double eps, int *resp
 
 }
 
-void dxPDF(double *t, double *a, double *v, double *w, double *sv, double eps, int *resp, int K, int N, int epsFLAG, double *da, double *dv, double *dw, int NThreads) {
+void dxPDF(double *t, double *a, double *v, double *w, double *sv, double eps, int *resp, int K, int N, int epsFLAG, double *da, double *dv, double *dw, double *dsv, int NThreads) {
 
   if (NThreads) {
     /* prepare threads */
@@ -575,6 +575,7 @@ void dxPDF(double *t, double *a, double *v, double *w, double *sv, double eps, i
           dadwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &da[i], eps, K, epsFLAG);
           dvdwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &dv[i]);
           dwdwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &dw[i], eps, K, epsFLAG);
+          dsvdwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &dsv[i], eps, K, epsFLAG);
         }
       });
     }
@@ -586,6 +587,7 @@ void dxPDF(double *t, double *a, double *v, double *w, double *sv, double eps, i
       dadwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &da[i], eps, K, epsFLAG);
       dvdwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &dv[i]);
       dwdwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &dw[i], eps, K, epsFLAG);
+      dsvdwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &dsv[i], eps, K, epsFLAG);
     }
 
     for (int j = 0; j < AmntOfThreads-1; j++) {
@@ -601,6 +603,7 @@ void dxPDF(double *t, double *a, double *v, double *w, double *sv, double eps, i
       dadwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &da[i], eps, K, epsFLAG);
       dvdwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &dv[i]);
       dwdwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &dw[i], eps, K, epsFLAG);
+      dsvdwiener(t[i]*pm, a[i], v[i], w[i], sv[i], ld, &dsv[i], eps, K, epsFLAG);
     }
   }
 
@@ -972,3 +975,58 @@ void dxCDF7(double *t, int *resp, double *a, double *v, double *t0, double *w, d
     }
 
   }
+/* ------------------------------------------------ */
+
+
+/* Quantile of Wiener diffusion */
+void quantile(double *t, double *a, double *v, double *w, double eps, int *resp, int K, int N, int epsFLAG, double *Rcdf, double *Rlogcdf, int NThreads) {
+  
+  if (NThreads) {
+    /* prepare threads */
+    int maxThreads = std::thread::hardware_concurrency();
+    if (maxThreads == 0) Rprintf("Could not find out number of threads. Taking 2 threads.\n");
+    int suppThreads = maxThreads == 0 ? 2 : maxThreads;
+    int AmntOfThreads = suppThreads > NThreads ? NThreads : suppThreads;
+    int NperThread = N / AmntOfThreads;
+    std::vector<std::thread> threads(AmntOfThreads-1);
+    
+    /* calculate derivative with parallelization */
+    for (int j = 0; j < AmntOfThreads-1; j++) {
+      threads[j] = std::thread([=]() {
+        for (int i = j*NperThread; i < (j+1)*NperThread; i++) {
+          // int pm = (resp[i]==1) ? 1 : -1;
+          // int mp = -1*pm;
+          // double lp = pwiener(t[i], a[i], v[i]*mp, pm*(resp[i]-w[i]), eps, K, epsFLAG);
+          // Rlogcdf[i] = lp;
+          // Rcdf[i] = exp(lp);
+        }
+      });
+    }
+    
+    int last = NperThread * (AmntOfThreads-1);
+    for (int i = last; i < N; i++) {
+      // int pm = (resp[i]==1) ? 1 : -1;
+      // int mp = -1*pm;
+      // double lp = pwiener(t[i], a[i], v[i]*mp, pm*(resp[i]-w[i]), eps, K, epsFLAG);
+      // Rlogcdf[i] = lp;
+      // Rcdf[i] = exp(lp);
+    }
+    
+    for (int j = 0; j < AmntOfThreads-1; j++) {
+      threads[j].join();
+    }
+    
+  } else {
+    /* calculate derivative without parallelization */
+    for(int i = 0; i < N; i++) {
+      // if (i % 1024 == 0) R_CheckUserInterrupt();
+      // int pm = (resp[i]==1) ? 1 : -1;
+      // int mp = -1*pm;
+      // double lp = pwiener(t[i], a[i], v[i]*mp, pm*(resp[i]-w[i]), eps, K, epsFLAG);
+      // Rlogcdf[i] = lp;
+      // Rcdf[i] = exp(lp);
+    }
+  }
+  
+}
+/* ------------------------------------------------ */
