@@ -2,14 +2,14 @@
 // Chair of Social Psychology, University of Freiburg
 // Authors: Christoph Klauer and Raphael Hartmann
 
-#include "tools.h"
-#include "cdf_fncs.h"
-#include "fncs_seven.h"
-#include "rwiener.h"
 #include <cmath>
 #include <thread>
 #include <mutex>
 #include <atomic>
+#include "cdf_fncs.h"
+#include "fncs_seven.h"
+#include "rwiener.h"
+#include "tools.h"
 
 
 std::mutex mtx_ars1, mtx_ars2; // mutexes for global ars_archiv's
@@ -315,7 +315,7 @@ void method1_both(int N, double a, double v, double w, double t0, double sv, dou
 			if (sv_or_sw) {
 				pdiff(0, bound, 1.0, a, v, 0, w, sw, sv, 0, err, K, epsFLAG, Neval, &p_up, &Rerr);
 			} else {
-				p_up = exp( logdiff(log(1), 2*v*a*w) - logdiff(-2*v*a*(1-w), 2*v*a*w) );
+				p_up = exp( logdiff(0, 2*v*a*w) - logdiff(-2*v*a*(1-w), 2*v*a*w) );
 				// = (1-exp(2*v*a*w))/(exp(-2*v*a*(1-w))-exp(2*v*a*w));
 			}
 		}
@@ -426,7 +426,7 @@ void method1_both(int N, double a, double v, double w, double t0, double sv, dou
 			} else {
 				// p_up = exp(pwiener(bound, a, -v, 1-w, err, K, epsFLAG));
 				//p_up = (1-exp(2*v*a*w))/(exp(-2*v*a*(1-w))-exp(2*v*a*w));
-				p_up = exp( logdiff(log(1), 2*v*a*w) - logdiff(-2*v*a*(1-w), 2*v*a*w) );
+				p_up = exp( logdiff(0, 2*v*a*w) - logdiff(-2*v*a*(1-w), 2*v*a*w) );
 				// = (1-exp(2*v*a*w))/(exp(-2*v*a*(1-w))-exp(2*v*a*w));
 			}
 		}
@@ -497,7 +497,8 @@ void method2_one(int N, double a, double v, double w, double t0, double sv, doub
               if (truncated) { // truncated
                 P = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
               } else { // not truncated
-                P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+                P = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+                // P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
               }
               REPEAT = oneuni() > P;
             }
@@ -530,7 +531,8 @@ void method2_one(int N, double a, double v, double w, double t0, double sv, doub
           if (truncated) { // truncated
             P = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
           } else { // not truncated
-            P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+            P = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+            // P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
           }
           REPEAT = oneuni() > P;
         }
@@ -569,7 +571,8 @@ void method2_one(int N, double a, double v, double w, double t0, double sv, doub
           if (truncated) { // truncated
             P = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
           } else { // not truncated
-            P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+            P = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+            // P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
           }
           REPEAT = oneuni() > P;
         }
@@ -742,7 +745,8 @@ void method3_one(int N, double a, double v, double w, double t0, double sv, doub
                 }
               } else { // not truncated
                 // p_lo = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
-                p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+                p_lo = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+                // p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
                 if (R == 1) REPEAT = oneuni() > p_lo;
                 if (R == 2) REPEAT = oneuni() > 1-p_lo;
               }
@@ -782,7 +786,8 @@ void method3_one(int N, double a, double v, double w, double t0, double sv, doub
             }
           } else { // not truncated
             // p_lo = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
-            p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+            p_lo = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+            // p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
             if (R == 1) REPEAT = oneuni() > p_lo;
             if (R == 2) REPEAT = oneuni() > 1-p_lo;
           }
@@ -828,7 +833,8 @@ void method3_one(int N, double a, double v, double w, double t0, double sv, doub
             }
           } else { // not truncated
             // p_lo = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
-            p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+            p_lo = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+            // p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
             if (R == 1) REPEAT = oneuni() > p_lo;
             if (R == 2) REPEAT = oneuni() > 1-p_lo;
           }
@@ -900,7 +906,8 @@ void method3_both(int N, double a, double v, double w, double t0, double sv, dou
             if (sv) vs += sv * onenorm();
         		if (sw) ws += sw * (oneuni()-0.5);
         		// p_lo = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
-        		p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+        		p_lo = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+        		// p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
         		up_or_down = oneuni() < p_lo ? 0 : 1;
           }
           double tau = 0;
@@ -951,7 +958,8 @@ void method3_both(int N, double a, double v, double w, double t0, double sv, dou
         if (sv) {vs += sv * onenorm();}
     		if (sw) {ws += sw * (oneuni()-0.5);}
     		// p_lo = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
-    		p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+    		p_lo = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+    		// p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
     		up_or_down = oneuni() < p_lo ? 0 : 1;
       }
       double tau = 0;
@@ -1008,7 +1016,8 @@ void method3_both(int N, double a, double v, double w, double t0, double sv, dou
         if (sv) {vs += sv * onenorm();}
     		if (sw) {ws += sw * (oneuni()-0.5);}
     		// p_lo = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
-    		p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+    		p_lo = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+    		// p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
     		up_or_down = oneuni() < p_lo ? 0 : 1;
       }
       double tau = 0;
@@ -1058,7 +1067,8 @@ void method4_one(int N, double a, double v, double w, double t0, double sv, doub
               if (truncated) { // truncated
                 P = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
               } else { // not truncated
-                P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+                P = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+                // P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
               }
               REPEAT = oneuni() > P;
             }
@@ -1098,7 +1108,8 @@ void method4_one(int N, double a, double v, double w, double t0, double sv, doub
           if (truncated) { // truncated
             P = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
           } else { // not truncated
-            P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+            P = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+            // P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
           }
           REPEAT = oneuni() > P;
         }
@@ -1144,7 +1155,8 @@ void method4_one(int N, double a, double v, double w, double t0, double sv, doub
           if (truncated) { // truncated
             P = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
           } else { // not truncated
-            P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+            P = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+            // P = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
           }
           REPEAT = oneuni() > P;
         }
@@ -1222,7 +1234,8 @@ void method4_both(int N, double a, double v, double w, double t0, double sv, dou
             if (sv) vs += sv * onenorm();
         		if (sw) ws += sw * (oneuni()-0.5);
         		// p_lo = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
-        		p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+        		p_lo = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+        		// p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
         		up_or_down = oneuni() < p_lo ? 0 : 1;
           }
 					ars_archiv ars_store_t;
@@ -1286,7 +1299,8 @@ void method4_both(int N, double a, double v, double w, double t0, double sv, dou
         if (sv) {vs += sv * onenorm();}
     		if (sw) {ws += sw * (oneuni()-0.5);}
     		// p_lo = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
-    		p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+    		p_lo = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+    		// p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
     		up_or_down = oneuni() < p_lo ? 0 : 1;
       }
 			ars_archiv ars_store_t;
@@ -1356,7 +1370,8 @@ void method4_both(int N, double a, double v, double w, double t0, double sv, dou
         if (sv) {vs += sv * onenorm();}
     		if (sw) {ws += sw * (oneuni()-0.5);}
     		// p_lo = exp(pwiener(bound, a, vs, ws, err, K, epsFLAG));
-    		p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
+    		p_lo = exp( logdiff(0, -2*vs*a*(1-ws)) - logdiff(2*vs*a*ws, -2*vs*a*(1-ws)) );
+    		// p_lo = (1-exp(-2*vs*a*(1-ws)))/(exp(2*vs*a*ws)-exp(-2*vs*a*(1-ws)));
     		up_or_down = oneuni() < p_lo ? 0 : 1;
       }
 			ars_archiv ars_store_t;
